=== modified file 'NEWS'
--- NEWS	2006-04-20 02:28:21 +0000
+++ NEWS	2006-04-20 02:42:17 +0000
@@ -80,6 +80,11 @@
     * 'merge' accepts --remember option to store parent location, like 'push'
       and 'pull'. (Olaf Conradi)
 
+    * bzr status and diff when files given as arguments do not exist
+      in the relevant trees.  (Martin Pool, #3619)
+
+    * Add '.hg' to the default ignore list.  (Martin Pool)
+
     * 'knit' is now the default disk format. This improves disk performance and
       utilization, increases incremental pull performance, robustness with SFTP
       and allows checkouts over SFTP to perform acceptable. 

=== modified file 'bzrlib/__init__.py'
--- bzrlib/__init__.py	2006-04-14 01:26:52 +0000
+++ bzrlib/__init__.py	2006-04-20 02:37:21 +0000
@@ -46,6 +46,7 @@
                   '.arch-inventory',
                   '.bzr.log',
                   '.del-*',
+                  '.hg',
                   '.git',
                   '.jamdeps'
                   '.libs',

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2006-04-20 02:28:21 +0000
+++ bzrlib/builtins.py	2006-04-20 02:42:17 +0000
@@ -931,10 +931,11 @@
     in the repository, not in the branch directory, if the branch format supports
     shared storage.
 
-    example:    
+    example:
         bzr init-repo repo
-        bzr init --format=metadir repo/trunk
-        cd repo/trunk
+        bzr init repo/trunk
+        bzr checkout --lightweight repo/trunk trunk-checkout
+        cd trunk-checkout
         (add files here)
     """
     takes_args = ["location"] 

=== modified file 'bzrlib/delta.py'
--- bzrlib/delta.py	2006-04-13 05:37:24 +0000
+++ bzrlib/delta.py	2006-04-19 07:16:05 +0000
@@ -160,7 +160,8 @@
         files within them.  Any unversioned files given have no effect
         (but this might change in the future).
     """
-
+    # NB: show_status depends on being able to pass in non-versioned files and
+    # report them as unknown
     old_tree.lock_read()
     try:
         new_tree.lock_read()

=== modified file 'bzrlib/diff.py'
--- bzrlib/diff.py	2006-04-19 02:16:05 +0000
+++ bzrlib/diff.py	2006-04-20 02:42:17 +0000
@@ -328,6 +328,25 @@
         raise errors.PathsNotVersionedError(sorted(unversioned))
     
 
+def _raise_if_nonexistent(paths, old_tree, new_tree):
+    """Complain if paths are not in either inventory or tree.
+
+    It's OK with the files exist in either tree's inventory, or 
+    if they exist in the tree but are not versioned.
+    
+    This can be used by operations such as bzr status that can accept
+    unknown or ignored files.
+    """
+    mutter("check paths: %r", paths)
+    if not paths:
+        return
+    s = old_tree.filter_unversioned_files(paths)
+    s = new_tree.filter_unversioned_files(s)
+    s = [path for path in s if not new_tree.has_filename(path)]
+    if s:
+        raise errors.PathsDoNotExist(sorted(s))
+
+
 def get_prop_change(meta_modified):
     if meta_modified:
         return " (properties changed)"

=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2006-04-20 02:28:21 +0000
+++ bzrlib/errors.py	2006-04-20 02:42:17 +0000
@@ -68,6 +68,10 @@
 # TODO: Convert all the other error classes here to BzrNewError, and eliminate
 # the old one.
 
+# TODO: The pattern (from hct) of using classes docstrings as message
+# templates is cute but maybe not such a great idea - perhaps should have a
+# separate static message_template.
+
 
 class BzrError(StandardError):
     def __str__(self):
@@ -136,7 +140,7 @@
 
 
 class NoWorkingTree(BzrNewError):
-    """No WorkingTree exists for %s(base)."""
+    """No WorkingTree exists for %(base)s."""
     
     def __init__(self, base):
         BzrNewError.__init__(self)
@@ -144,7 +148,7 @@
 
 
 class NotLocalUrl(BzrNewError):
-    """%s(url) is not a local path."""
+    """%(url)s is not a local path."""
     
     def __init__(self, url):
         BzrNewError.__init__(self)
@@ -172,6 +176,8 @@
     """Commit refused because there are unknowns in the tree."""
 
 
+# XXX: Should be unified with TransportError; they seem to represent the
+# same thing
 class PathError(BzrNewError):
     """Generic path error: %(path)r%(extra)s)"""
 
@@ -280,6 +286,20 @@
         self.paths_as_string = ' '.join([quotefn(p) for p in paths])
 
 
+class PathsDoNotExist(BzrNewError):
+    """Path(s) do not exist: %(paths_as_string)s"""
+
+    # used when reporting that paths are neither versioned nor in the working
+    # tree
+
+    def __init__(self, paths):
+        # circular import
+        from bzrlib.osutils import quotefn
+        BzrNewError.__init__(self)
+        self.paths = paths
+        self.paths_as_string = ' '.join([quotefn(p) for p in paths])
+
+
 class BadFileKindError(BzrError):
     """Specified file is of a kind that cannot be added.
 

=== modified file 'bzrlib/help.py'
--- bzrlib/help.py	2006-02-10 00:39:25 +0000
+++ bzrlib/help.py	2006-04-19 07:20:40 +0000
@@ -26,8 +26,7 @@
 """Bazaar-NG -- a free distributed version-control tool
 http://bazaar-ng.org/
 
-WARNING: This is an unstable development version.
-         Please keep backups.
+Please remember to keep backups!
 
 Basic commands:
 

=== modified file 'bzrlib/inventory.py'
--- bzrlib/inventory.py	2006-04-20 02:28:21 +0000
+++ bzrlib/inventory.py	2006-04-20 02:42:17 +0000
@@ -1,4 +1,4 @@
-# (C) 2005 Canonical Ltd
+# Copyright (C) 2005, 2006 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -349,18 +349,36 @@
         raise BzrCheckError('unknown entry kind %r in revision {%s}' % 
                             (self.kind, rev_id))
 
-
     def copy(self):
         """Clone this inventory entry."""
         raise NotImplementedError
 
-    def _get_snapshot_change(self, previous_entries):
+    def _describe_snapshot_change(self, previous_entries):
+        """Describe how this entry will have changed in a new commit.
+
+        :param previous_entries: Dictionary from revision_id to inventory entry.
+
+        :returns: One-word description: "merged", "added", "renamed", "modified".
+        """
+        # XXX: This assumes that the file *has* changed -- it should probably
+        # be fused with whatever does that detection.  Why not just a single
+        # thing to compare the entries?
+        #
+        # TODO: Return some kind of object describing all the possible
+        # dimensions that can change, not just a string.  That can then give
+        # both old and new names for renames, etc.
+        #
         if len(previous_entries) > 1:
             return 'merged'
         elif len(previous_entries) == 0:
             return 'added'
-        else:
-            return 'modified/renamed/reparented'
+        the_parent, = previous_entries.values()
+        if self.parent_id != the_parent.parent_id:
+            # actually, moved to another directory
+            return 'renamed'
+        elif self.name != the_parent.name:
+            return 'renamed'
+        return 'modified'
 
     def __repr__(self):
         return ("%s(%r, %r, parent_id=%r)"
@@ -385,15 +403,23 @@
                 mutter("found unchanged entry")
                 self.revision = parent_ie.revision
                 return "unchanged"
-        return self.snapshot_revision(revision, previous_entries, 
-                                      work_tree, weave_store, transaction)
-
-    def snapshot_revision(self, revision, previous_entries, work_tree,
-                          weave_store, transaction):
-        """Record this revision unconditionally."""
-        mutter('new revision for {%s}', self.file_id)
+        return self._snapshot_into_revision(revision, previous_entries, 
+                                            work_tree, weave_store, transaction)
+
+    def _snapshot_into_revision(self, revision, previous_entries, work_tree,
+                                weave_store, transaction):
+        """Record this revision unconditionally into a store.
+
+        The entry's last-changed revision property (`revision`) is updated to 
+        that of the new revision.
+        
+        :param revision: id of the new revision that is being recorded.
+
+        :returns: String description of the commit (e.g. "merged", "modified"), etc.
+        """
+        mutter('new revision {%s} for {%s}', revision, self.file_id)
         self.revision = revision
-        change = self._get_snapshot_change(previous_entries)
+        change = self._describe_snapshot_change(previous_entries)
         self._snapshot_text(previous_entries, work_tree, weave_store,
                             transaction)
         return change

=== modified file 'bzrlib/lockdir.py'
--- bzrlib/lockdir.py	2006-04-18 22:41:16 +0000
+++ bzrlib/lockdir.py	2006-04-20 02:42:17 +0000
@@ -109,9 +109,11 @@
         LockError,
         LockNotHeld,
         NoSuchFile,
+        PathError,
         ResourceBusy,
         UnlockableTransport,
         )
+from bzrlib.trace import mutter
 from bzrlib.transport import Transport
 from bzrlib.osutils import rand_chars
 from bzrlib.rio import RioWriter, read_stanza, Stanza
@@ -201,11 +203,9 @@
             self.transport.rename(tmpname, self._held_dir)
             self._lock_held = True
             self.confirm()
-            return
-        except (DirectoryNotEmpty, FileExists, ResourceBusy), e:
-            pass
-        # fall through to here on contention
-        raise LockContention(self)
+        except (PathError, DirectoryNotEmpty, FileExists, ResourceBusy), e:
+            mutter("contention on %r: %s", self, e)
+            raise LockContention(self)
 
     def unlock(self):
         """Release a held lock

=== modified file 'bzrlib/status.py'
--- bzrlib/status.py	2006-04-05 20:37:37 +0000
+++ bzrlib/status.py	2006-04-19 07:16:05 +0000
@@ -1,4 +1,4 @@
-# (C) 2005 Canonical Ltd
+# Copyright (C) 2005, 2006 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,6 +17,7 @@
 import sys
 
 from bzrlib.delta import compare_trees
+from bzrlib.diff import _raise_if_nonexistent
 from bzrlib.errors import NoSuchRevision
 from bzrlib.log import line_log
 from bzrlib.osutils import is_inside_any
@@ -83,11 +84,16 @@
     The result is written out as Unicode and to_file should be able 
     to encode that.
 
+    If showing the status of a working tree, extra information is included
+    about unknown files, conflicts, and pending merges.
+
     show_unchanged
         If set, includes unchanged files.
 
     specific_files
-        If set, only show the status of files in this list.
+        If set, a list of filenames whose status should be shown.
+        It is an error to give a filename that is not in the working 
+        tree, or in the working inventory or in the basis inventory.
 
     show_ids
         If set, includes each file's id.
@@ -127,7 +133,7 @@
                     raise BzrCommandError(str(e))
             else:
                 new = wt
-                
+        _raise_if_nonexistent(specific_files, old, new)
         delta = compare_trees(old, new, want_unchanged=show_unchanged,
                               specific_files=specific_files)
         delta.show(to_file,

=== modified file 'bzrlib/tests/blackbox/test_commit.py'
--- bzrlib/tests/blackbox/test_commit.py	2006-03-20 20:02:21 +0000
+++ bzrlib/tests/blackbox/test_commit.py	2006-04-19 05:11:59 +0000
@@ -17,14 +17,12 @@
 
 """Tests for the commit CLI of bzr."""
 
-from cStringIO import StringIO
 import os
 import re
-import shutil
 import sys
 
 from bzrlib.branch import Branch
-import bzrlib.bzrdir as bzrdir
+from bzrlib.bzrdir import BzrDir
 from bzrlib.errors import BzrCommandError
 from bzrlib.tests.blackbox import ExternalBase
 from bzrlib.workingtree import WorkingTree
@@ -50,12 +48,54 @@
                          'Committed revision 1.\n',
                          err)
 
-    def test_15_verbose_commit_with_unknown(self):
+    def prepare_simple_history(self):
+        """Prepare and return a working tree with one commit of one file"""
+        # Commit with modified file should say so
+        wt = BzrDir.create_standalone_workingtree('.')
+        self.build_tree(['hello.txt', 'extra.txt'])
+        wt.add(['hello.txt'])
+        wt.commit(message='added')
+        return wt
+
+    def test_verbose_commit_modified(self):
+        # Verbose commit of modified file should say so
+        wt = self.prepare_simple_history()
+        self.build_tree_contents([('hello.txt', 'new contents')])
+        out, err = self.run_bzr("commit", "-m", "modified")
+        self.assertEqual('', out)
+        self.assertEqual('modified hello.txt\n'
+                         'Committed revision 2.\n',
+                         err)
+
+    def test_verbose_commit_renamed(self):
+        # Verbose commit of renamed file should say so
+        wt = self.prepare_simple_history()
+        wt.rename_one('hello.txt', 'gutentag.txt')
+        out, err = self.run_bzr("commit", "-m", "renamed")
+        self.assertEqual('', out)
+        self.assertEqual('renamed gutentag.txt\n'
+                         'Committed revision 2.\n',
+                         err)
+
+    def test_verbose_commit_moved(self):
+        # Verbose commit of file moved to new directory should say so
+        wt = self.prepare_simple_history()
+        os.mkdir('subdir')
+        wt.add(['subdir'])
+        wt.rename_one('hello.txt', 'subdir/hello.txt')
+        out, err = self.run_bzr("commit", "-m", "renamed")
+        self.assertEqual('', out)
+        self.assertEqualDiff('added subdir\n'
+                             'renamed subdir/hello.txt\n'
+                             'Committed revision 2.\n',
+                             err)
+
+    def test_verbose_commit_with_unknown(self):
         """Unknown files should not be listed by default in verbose output"""
         # Is that really the best policy?
-        self.runbzr("init")
+        wt = BzrDir.create_standalone_workingtree('.')
         self.build_tree(['hello.txt', 'extra.txt'])
-        self.runbzr("add hello.txt")
+        wt.add(['hello.txt'])
         out,err = self.run_bzr("commit", "-m", "added")
         self.assertEqual('', out)
         self.assertEqual('added hello.txt\n'

=== modified file 'bzrlib/tests/blackbox/test_status.py'
--- bzrlib/tests/blackbox/test_status.py	2006-02-23 01:34:20 +0000
+++ bzrlib/tests/blackbox/test_status.py	2006-04-20 02:24:22 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005 by Canonical Ltd
+# Copyright (C) 2005, 2006 by Canonical Ltd
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -32,6 +32,7 @@
 import bzrlib.branch
 from bzrlib.builtins import merge
 import bzrlib.bzrdir as bzrdir
+import bzrlib.errors as errors
 from bzrlib.osutils import pathjoin
 from bzrlib.revisionspec import RevisionSpec
 from bzrlib.status import show_tree_status
@@ -154,14 +155,10 @@
                            '  directory/hello.c\n'
                            ])
 
-        tof = StringIO()
-        show_tree_status(wt, specific_files=['bye.c','test.c','absent.c'], 
-                         to_file=tof)
-        tof.seek(0)
-        self.assertEquals(tof.readlines(),
-                          ['unknown:\n',
-                           '  bye.c\n'
-                           ])
+        self.assertRaises(errors.PathsDoNotExist,
+                          show_tree_status,
+                          wt, specific_files=['bye.c','test.c','absent.c'], 
+                          to_file=tof)
         
         tof = StringIO()
         show_tree_status(wt, specific_files=['directory'], to_file=tof)
@@ -178,6 +175,14 @@
                            '  dir2\n'
                            ])
 
+    def test_status_nonexistent_file(self):
+        # files that don't exist in either the basis tree or working tree
+        # should give an error
+        wt = self.make_branch_and_tree('.')
+        out, err = self.run_bzr('status', 'does-not-exist', retcode=3)
+        self.assertContainsRe(err, r'do not exist.*does-not-exist')
+
+
 class CheckoutStatus(BranchStatus):
 
     def setUp(self):

=== modified file 'bzrlib/tests/test_transport_implementations.py'
--- bzrlib/tests/test_transport_implementations.py	2006-04-19 23:32:08 +0000
+++ bzrlib/tests/test_transport_implementations.py	2006-04-20 02:42:17 +0000
@@ -788,9 +788,9 @@
         self.assertEqual([u'a', u'c', u'c2'], sorted_list('.'))
         self.assertEqual([u'e'], sorted_list(u'c'))
 
-        self.assertListRaises(NoSuchFile, t.list_dir, 'q')
-        self.assertListRaises(NoSuchFile, t.list_dir, 'c/f')
-        self.assertListRaises(NoSuchFile, t.list_dir, 'a')
+        self.assertListRaises(PathError, t.list_dir, 'q')
+        self.assertListRaises(PathError, t.list_dir, 'c/f')
+        self.assertListRaises(PathError, t.list_dir, 'a')
 
     def test_clone(self):
         # TODO: Test that clone moves up and down the filesystem

=== modified file 'bzrlib/transport/sftp.py'
--- bzrlib/transport/sftp.py	2006-04-19 23:32:08 +0000
+++ bzrlib/transport/sftp.py	2006-04-20 02:42:17 +0000
@@ -1,4 +1,5 @@
-# Copyright (C) 2005 Robey Pointer <robey@lag.net>, Canonical Ltd
+# Copyright (C) 2005 Robey Pointer <robey@lag.net>
+# Copyright (C) 2005, 2006 Canonical Ltd
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -34,7 +35,9 @@
                            FileExists, 
                            TransportNotPossible, NoSuchFile, PathNotChild,
                            TransportError,
-                           LockError, ParamikoNotPresent
+                           LockError, 
+                           PathError,
+                           ParamikoNotPresent,
                            )
 from bzrlib.osutils import pathjoin, fancy_rename
 from bzrlib.trace import mutter, warning, error
@@ -477,7 +480,8 @@
             self._translate_io_exception(e, path, ': unable to mkdir',
                 failure_exc=FileExists)
 
-    def _translate_io_exception(self, e, path, more_info='', failure_exc=NoSuchFile):
+    def _translate_io_exception(self, e, path, more_info='', 
+                                failure_exc=PathError):
         """Translate a paramiko or IOError into a friendlier exception.
 
         :param e: The original exception
@@ -487,8 +491,8 @@
         :param failure_exc: Paramiko has the super fun ability to raise completely
                            opaque errors that just set "e.args = ('Failure',)" with
                            no more information.
-                           This sometimes means FileExists, but it also sometimes
-                           means NoSuchFile
+                           If this parameter is set, it defines the exception 
+                           to raise in these cases.
         """
         # paramiko seems to generate detailless errors.
         self._translate_error(e, path, raise_generic=False)

=== modified file 'tutorial.txt'
--- tutorial.txt	2006-04-12 04:57:34 +0000
+++ tutorial.txt	2006-04-19 05:28:51 +0000
@@ -2,7 +2,8 @@
 .. into HTML or text.  In the future we plan to extract the example commands
 .. and automatically test them.
 
-.. This text was previously on the wiki at http://bazaar.canonical.com/IntroductionToBzr
+.. This text was previously on the wiki at 
+.. http://bazaar.canonical.com/IntroductionToBzr
 .. but has been moved into the source tree so it can be kept in sync with
 .. the source and possibly automatically checked.
 
@@ -10,49 +11,111 @@
 Bazaar-NG Tutorial
 ==================
 
-Current for bzr-0.7pre, 2006-01-06.
+Current for bzr-0.8, 2006-04
 
 
 Introduction
 ============
 
-If you are already familiar with decentralized revision control, then please feel free to skip ahead to "Introducing Yourself to Bazaar-NG". If, on the other hand, you are familiar with revision control but not decentralized revision control, then please start at "How DRCS is different." Otherwise, get some coffee or tea, get comfortable and get ready to catch up. 
+If you are already familiar with decentralized revision control, then
+please feel free to skip ahead to "Introducing Yourself to Bazaar-NG". If,
+on the other hand, you are familiar with revision control but not
+decentralized revision control, then please start at "How DRCS is
+different." Otherwise, get some coffee or tea, get comfortable and get
+ready to catch up. 
 
 The Purposes of Revision Control
 ================================
 
-Odds are that you have worked on some sort of textual data -- the sources to a program, web sites or the config files that Unix system administrators have to deal with in /etc. The chances are also good that you have made some sort of mistake that you deeply regretted. Perhaps you deleted the configuration file for your mailserver or perhaps mauled the source code for a pet project. Whatever happened, you have just deleted important information that you would desperately like to get back. If this has ever happened to you, then you are probably ready for Bazaar-NG.
-
-Revision control systems (which I'll henceforth call RCS) such as Bazaar-NG give you the ability to track changes for a directory by turning it into something slightly more complicated than a directory that we call a **branch**. The branch not only stores how the directory looks right now, but also how it looked at various points in the past. Then, when you do something you wish you hadn't, you can restore the directory to the way it looked at some point in the past.
-
-Revision control systems give users the ability to save changes to a branch by "committing a **revision**". The revision created is essentially a summary of the changes that were made since the last time the tree was saved. 
-
-These revisions have other uses as well. For example, one can comment revisions to record what the recent set of changes meant by providing an optional log message. Real life log messages include things like "Fixed the web template to close the table" and "Added sftp suppport. Fixes #595"
+Odds are that you have worked on some sort of textual data -- the sources
+to a program, web sites or the config files that Unix system
+administrators have to deal with in /etc. The chances are also good that
+you have made some sort of mistake that you deeply regretted. Perhaps you
+deleted the configuration file for your mailserver or perhaps mauled the
+source code for a pet project. Whatever happened, you have just deleted
+important information that you would desperately like to get back. If this
+has ever happened to you, then you are probably ready for Bazaar-NG.
+
+Revision control systems (which I'll henceforth call RCS) such as
+Bazaar-NG give you the ability to track changes for a directory by turning
+it into something slightly more complicated than a directory that we call
+a **branch**. The branch not only stores how the directory looks right
+now, but also how it looked at various points in the past. Then, when you
+do something you wish you hadn't, you can restore the directory to the way
+it looked at some point in the past.
+
+Revision control systems give users the ability to save changes to a
+branch by "committing a **revision**". The revision created is essentially
+a summary of the changes that were made since the last time the tree was
+saved. 
+
+These revisions have other uses as well. For example, one can comment
+revisions to record what the recent set of changes meant by providing an
+optional log message. Real life log messages include things like "Fixed
+the web template to close the table" and "Added sftp suppport. Fixes #595"
 	
-We keep these logs so that if later there is some sort of problem with sftp, we can figure out when the problem probably happened. 
+We keep these logs so that if later there is some sort of problem with
+sftp, we can figure out when the problem probably happened. 
 
 How DRCS is Different
 ---------------------
 
-Many Revision Control Systems (RCS) are stored on servers. If one wants to work on the code stored within an RCS, then one needs to connect to the server and "checkout" the code. Doing so gives one a directory in which a person can make changes and then commit. The RCS client then connects to the RCS server and stores the changes. This method is known as the centralized model. 
-
-The centralized model can have some drawbacks. A centralized RCS requires that one is able to connect to the server whenever one wants to do version control work. This can be a bit of a problem if your server on some other machine on the internet and you are not. Or, worse yet, you ''are'' on the internet but the server is missing!
-
-Decentralized Revision Control Systems (which I'll call DRCS after this point) deal with this problem by keeping branches on the same machine as the client. In Bazaar-NG's case, the branch is kept in the same place as the code that is being version controlled. This allows the user to save his changes (**commit**) whenever he wants -- even if he is offline. The user only needs internet access when he wants to access the changes in someone else's branch that are somewhere else.
+Many Revision Control Systems (RCS) are stored on servers. If one wants to
+work on the code stored within an RCS, then one needs to connect to the
+server and "checkout" the code. Doing so gives one a directory in which a
+person can make changes and then commit. The RCS client then connects to
+the RCS server and stores the changes. This method is known as the
+centralized model. 
+
+The centralized model can have some drawbacks. A centralized RCS requires
+that one is able to connect to the server whenever one wants to do version
+control work. This can be a bit of a problem if your server on some other
+machine on the internet and you are not. Or, worse yet, you ''are'' on the
+internet but the server is missing!
+
+Decentralized Revision Control Systems (which I'll call DRCS after this
+point) deal with this problem by keeping branches on the same machine as
+the client. In Bazaar-NG's case, the branch is kept in the same place as
+the code that is being version controlled. This allows the user to save
+his changes (**commit**) whenever he wants -- even if he is offline. The
+user only needs internet access when he wants to access the changes in
+someone else's branch that are somewhere else.
 
  
-A common requirement that many people have is the need to keep track of the changes for a directory such as file and subdirectory changes. Performing this tracking by hand is a awkward process that over time becomes unwieldy. That is, until one considers version control tools such as Bazaar-NG. These tools automate the process of storing data by creating a **revision** of the directory tree whenever the user asks. 
-
-Revision control software such as Bazaar-NG can do much more than just storage and performing undo.  For example, with Bazaar-NG developer can take the modifications in one branch of software and apply them to another, related, branch -- even if those changes exist in a branch owned by somebody else. This allows developers to cooperate without giving write access to repository.
-
-Bazaar-NG remembers the ''ancestry'' of a revision: the previous revisions that it is based upon.  A single revision may have more than one direct descendant, each with different changes, representing a divergence in the evolution of the tree. By branching, Bazaar-NG allows multiple people to cooperate on the evolution of a project, without all needing to work in strict lock-step.  Branching can be useful even for a single developer.
+A common requirement that many people have is the need to keep track of
+the changes for a directory such as file and subdirectory changes.
+Performing this tracking by hand is a awkward process that over time
+becomes unwieldy. That is, until one considers version control tools such
+as Bazaar-NG. These tools automate the process of storing data by creating
+a **revision** of the directory tree whenever the user asks. 
+
+Revision control software such as Bazaar-NG can do much more than just
+storage and performing undo.  For example, with Bazaar-NG developer can
+take the modifications in one branch of software and apply them to
+another, related, branch -- even if those changes exist in a branch owned
+by somebody else. This allows developers to cooperate without giving write
+access to repository.
+
+Bazaar-NG remembers the ''ancestry'' of a revision: the previous revisions
+that it is based upon.  A single revision may have more than one direct
+descendant, each with different changes, representing a divergence in the
+evolution of the tree. By branching, Bazaar-NG allows multiple people to
+cooperate on the evolution of a project, without all needing to work in
+strict lock-step.  Branching can be useful even for a single developer.
 
 Introducing yourself to Bazaar-NG
 =================================
 
-Bazaar-NG installs a single new command, **bzr**.  Everything else is a subcommand of this.  You can get some help with `bzr help`.  There will be more in the future.
+Bazaar-NG installs a single new command, **bzr**.  Everything else is a
+subcommand of this.  You can get some help with `bzr help`.  There will be
+more in the future.
 
-One function of a version control system is to keep track of who changed what.  In a decentralized system, that requires an identifier for each author that is globally unique.  Most people already have one of these: an email address. Bzr is smart enough to automatically generate an email address by looking up your username and hostname. If you don't like the guess that Bazaar-NG makes, then three options exist:
+One function of a version control system is to keep track of who changed
+what.  In a decentralized system, that requires an identifier for each
+author that is globally unique.  Most people already have one of these: an
+email address. Bzr is smart enough to automatically generate an email
+address by looking up your username and hostname. If you don't like the
+guess that Bazaar-NG makes, then three options exist:
 
  1. (**Bazaar-NG 0.6 and later**). Setting the email address in the
  ``~/.bazaar/bazaar.conf`` by adding the following lines. Please note that
@@ -69,12 +132,16 @@
     email=Your Name <email@isp.com>
  
  1. Overriding the two previous options by setting the global environment
- variable ``$BZREMAIL`` or ``$EMAIL`` (``$BZREMAIL`` will take precedence) to your full email address.
+ variable ``$BZREMAIL`` or ``$EMAIL`` (``$BZREMAIL`` will take precedence) 
+ to your full email address.
 
 Creating a branch
 =================
 
-History is by default stored in the .bzr directory of the branch. There will be a facility to store it in a separate repository, which may be remote.  We create a new branch by running **bzr init** in an existing directory::
+History is by default stored in the .bzr directory of the branch. There
+will be a facility to store it in a separate repository, which may be
+remote.  We create a new branch by running **bzr init** in an existing
+directory::
 
     % mkdir tutorial
     % cd tutorial
@@ -88,7 +155,9 @@
     ./  ../  .bzr/
     %
 
-As for CVS, there are three classes of file: unknown, ignored, and versioned.  The **add** command makes a file versioned: that is, changes to it will be recorded by the system::
+As for CVS, there are three classes of file: unknown, ignored, and
+versioned.  The **add** command makes a file versioned: that is, changes
+to it will be recorded by the system::
 
     % echo 'hello world' > hello.txt
     % bzr status
@@ -101,12 +170,18 @@
     % bzr unknowns
 
 
-If you add the wrong file, simply use **bzr remove** to make it unversioned again.  This does not delete the working copy.
+If you add the wrong file, simply use **bzr remove** to make it
+unversioned again.  This does not delete the working copy.
 
 Branch locations
 ================
 
-All history is stored in a branch, which is just an on-disk directory containing control files.  By default there is no separate repository or database as used in svn or svk. You can choose to create a repository if you want to (see the **bzr init-repo** command). You may wish to do this if you have very large branches, or many branches of a moderate sized project.
+All history is stored in a branch, which is just an on-disk directory
+containing control files.  By default there is no separate repository or
+database as used in svn or svk. You can choose to create a repository if
+you want to (see the **bzr init-repo** command). You may wish to do this
+if you have very large branches, or many branches of a moderate sized
+project.
 
 You'll usually refer to branches on your computer's filesystem just by
 giving the name of the directory containing the branch.  bzr also supports
@@ -114,12 +189,20 @@
 
     % bzr log http://bazaar-ng.org/bzr/bzr.dev/
 
-By installing bzr plugins you can also access branches over the sftp or rsync protocols.
+By installing bzr plugins you can also access branches over the sftp or
+rsync protocols.
 
 Reviewing changes
 =================
 
-Once you have completed some work, you will want to **commit** it to the version history.  It is good to commit fairly often: whenever you get a new feature working, fix a bug, or improve some code or documentation.  It's also a good practice to make sure that the code compiles and passes its test suite before committing, to make sure that every revision is a known-good state.  You can also review your changes, to make sure you're committing what you intend to, and as a chance to rethink your work before you permanently record it. 
+Once you have completed some work, you will want to **commit** it to the
+version history.  It is good to commit fairly often: whenever you get a
+new feature working, fix a bug, or improve some code or documentation.
+It's also a good practice to make sure that the code compiles and passes
+its test suite before committing, to make sure that every revision is a
+known-good state.  You can also review your changes, to make sure you're
+committing what you intend to, and as a chance to rethink your work before
+you permanently record it. 
 
 Two bzr commands are particularly useful here: **status** and **diff**.  
 
@@ -133,12 +216,16 @@
     modified:
        foo
 
-By default **bzr status** hides "boring" files that are either unchanged or ignored.  To see them too, use the --all option.  The status command can optionally be given the name of some files or directories to check.
+By default **bzr status** hides "boring" files that are either unchanged
+or ignored.  To see them too, use the --all option.  The status command
+can optionally be given the name of some files or directories to check.
 
 bzr diff
 --------
 
-The **diff** command shows the full text of changes to all files as a standard unified diff.  This can be piped through many programs such as ''patch'', ''diffstat'', ''filterdiff'' and ''colordiff''::
+The **diff** command shows the full text of changes to all files as a
+standard unified diff.  This can be piped through many programs such as
+''patch'', ''diffstat'', ''filterdiff'' and ''colordiff''::
 
     % bzr diff
     *** added file 'hello.txt'
@@ -162,23 +249,36 @@
 Committing changes
 ==================
 
-When the working tree state is satisfactory, it can be **committed** to the branch, creating a new revision holding a snapshot of that state.  
+When the working tree state is satisfactory, it can be **committed** to
+the branch, creating a new revision holding a snapshot of that state.  
 
 bzr commit
 ----------
 
-The **commit** command takes a message describing the changes in the revision.  It also records your userid, the current time and timezone, and the inventory and contents of the tree.  The commit message is specified by the ''-m'' or ''--message'' option. You can enter a multi-line commit message; in most shells you can enter this just by leaving the quotes open at the end of the line.
+The **commit** command takes a message describing the changes in the
+revision.  It also records your userid, the current time and timezone, and
+the inventory and contents of the tree.  The commit message is specified
+by the ''-m'' or ''--message'' option. You can enter a multi-line commit
+message; in most shells you can enter this just by leaving the quotes open
+at the end of the line.
 
 ::
 
     % bzr commit -m "added my first file"
 
-You can also use the -F option to take the message from a file.  Some people like to make notes for a commit message while they work, then review the diff to make sure they did what they said they did.  (This file can also be useful when you pick up your work after a break.)
+You can also use the -F option to take the message from a file.  Some
+people like to make notes for a commit message while they work, then
+review the diff to make sure they did what they said they did.  (This file
+can also be useful when you pick up your work after a break.)
 
 Message from an editor
 ======================
 
-If you use neither the `-m` nor the `-F` option then bzr will open an editor for you to enter a message.  The editor to run is controlled by your `$EDITOR` environment variable or (Post Bazaar-NG 0.6) email setting in .  If you quit the editor without making any changes, the commit will be cancelled.
+If you use neither the `-m` nor the `-F` option then bzr will open an
+editor for you to enter a message.  The editor to run is controlled by
+your `$EDITOR` environment variable or (Post Bazaar-NG 0.6) email setting
+in .  If you quit the editor without making any changes, the commit will
+be cancelled.
 
 Selective commit
 ----------------
@@ -197,14 +297,24 @@
 Removing uncommitted changes
 ============================
 
-If you've made some changes and don't want to keep them, use the **revert** command to go back to the previous head version.  It's a good idea to use **bzr diff** first to see what will be removed. By default the revert command reverts the whole tree; if file or directory names are given then only those ones will be affected. **revert** also clears the list of pending merges revisions.
+If you've made some changes and don't want to keep them, use the
+**revert** command to go back to the previous head version.  It's a good
+idea to use **bzr diff** first to see what will be removed. By default the
+revert command reverts the whole tree; if file or directory names are
+given then only those ones will be affected. **revert** also clears the
+list of pending merges revisions.
 
 Ignoring files
 ==============
 
-Many source trees contain some files that do not need to be versioned, such as editor backups, object or bytecode files, and built programs.  You can simply not add them, but then they'll always crop up as unknown files.  You can also tell bzr to ignore these files by adding them to a file called ''.bzrignore'' at the top of the tree.
+Many source trees contain some files that do not need to be versioned,
+such as editor backups, object or bytecode files, and built programs.  You
+can simply not add them, but then they'll always crop up as unknown files.
+You can also tell bzr to ignore these files by adding them to a file
+called ''.bzrignore'' at the top of the tree.
 
-This file contains a list of file wildcards (or "globs"), one per line.  Typical contents are like this::
+This file contains a list of file wildcards (or "globs"), one per line.
+Typical contents are like this::
 
     *.o
     *~
@@ -220,15 +330,20 @@
     ./config.h
     doc/*.html
 
-To get a list of which files are ignored and what pattern they matched, use ''bzr ignored''::
+To get a list of which files are ignored and what pattern they matched,
+use ''bzr ignored''::
 
     % bzr ignored
     config.h                 ./config.h
     configure.in~            *~
 
-It is OK to have either an ignore pattern match a versioned file, or to add an ignored file.  Ignore patterns have no effect on versioned files; they only determine whether unversioned files are reported as unknown or ignored.
+It is OK to have either an ignore pattern match a versioned file, or to
+add an ignored file.  Ignore patterns have no effect on versioned files;
+they only determine whether unversioned files are reported as unknown or
+ignored.
 
-The ''.bzrignore'' file should normally be versioned, so that new copies of the branch see the same patterns::
+The ''.bzrignore'' file should normally be versioned, so that new copies
+of the branch see the same patterns::
 
     % bzr add .bzrignore
     % bzr commit -m "Add ignore patterns"
@@ -240,12 +355,14 @@
 bzr log
 -------
 
-The **bzr log** command shows a list of previous revisions. The **bzr log --forward** command does the same in chronological order to get most recent revisions printed at last.
+The **bzr log** command shows a list of previous revisions. The **bzr log
+--forward** command does the same in chronological order to get most
+recent revisions printed at last.
 
 As with bzr diff, bzr log supports the -r argument::
 
     % bzr log -r 1000..          # Revision 1000 and everything after it
-    % bzr log -r ..1000          # Everything up to and including revision % 1000
+    % bzr log -r ..1000          # Everything up to and including r1000
     % bzr log -r 1000..1100      # changes from 1000 to 1100
     % bzr log -r 1000            # The changes in only revision 1000
 
@@ -253,13 +370,15 @@
 Branch statistics
 =================
 
-The **bzr info** command shows some summary information about the working tree and the branch history.  
+The **bzr info** command shows some summary information about the working
+tree and the branch history.  
 
 
 Versioning directories
 ======================
 
-bzr versions files and directories in a way that can keep track of renames and intelligently merge them::
+bzr versions files and directories in a way that can keep track of renames
+and intelligently merge them::
 
     % mkdir src
     % echo 'int main() {}' > src/simple.c
@@ -275,9 +394,14 @@
 
 Deleting and removing files
 ===========================
-You can delete files or directories by just deleting them from the working directory.  This is a bit different to CVS, which requires that you also do **cvs remove**.
-
-	**bzr remove** makes the file un-versioned, but does not delete the working copy.  This is useful when you add the wrong file, or decide that a file should actually not be versioned. 
+
+You can delete files or directories by just deleting them from the working
+directory.  This is a bit different to CVS, which requires that you also
+do **cvs remove**.
+
+    **bzr remove** makes the file un-versioned, but does not delete
+    the working copy.  This is useful when you add the wrong file, or decide
+    that a file should actually not be versioned. 
 
 ::
 
@@ -289,23 +413,31 @@
     D       src/
     D       src/simple.c
 
-If you remove the wrong file by accident, you can use **bzr revert** to restore it.
+If you remove the wrong file by accident, you can use **bzr revert** to
+restore it.
 
 
 Branching
 =========
 
-Often rather than starting your own project, you will want to submit a change to an existing project.  You can get a copy of an existing branch by copying its directory, expanding a tarball, or by a remote copy using something like rsync.  You can also use bzr to fetch a copy.  Because this new copy is potentially a new branch, the command is called *branch*::
+Often rather than starting your own project, you will want to submit a
+change to an existing project.  You can get a copy of an existing branch
+by copying its directory, expanding a tarball, or by a remote copy using
+something like rsync.  You can also use bzr to fetch a copy.  Because this
+new copy is potentially a new branch, the command is called *branch*::
 
     % bzr branch http://bazaar-ng.org/bzr/bzr.dev 
     % cd bzr.dev
 
-This copies down the complete history of this branch, so we can do all operations on it locally: log, annotate, making and merging branches.  There will be an option to get only part of the history if you wish.
+This copies down the complete history of this branch, so we can do all
+operations on it locally: log, annotate, making and merging branches.
+There will be an option to get only part of the history if you wish.
 
 Following upstream changes
 ==========================
 
-You can stay up-to-date with the parent branch by "pulling" in their changes::
+You can stay up-to-date with the parent branch by "pulling" in their
+changes::
 
     % bzr pull
 
@@ -321,24 +453,43 @@
 Merging from related branches
 =============================
 
-If two branches have diverged (both have unique changes) then **bzr merge** is the appropriate command to use. Merge will automatically calculate the changes that exist in the branch you're merging from that are not in your branch and attempt to apply them in your branch.
+If two branches have diverged (both have unique changes) then **bzr
+merge** is the appropriate command to use. Merge will automatically
+calculate the changes that exist in the branch you're merging from that
+are not in your branch and attempt to apply them in your branch.
 
 ::
 
   % bzr merge URL
 
 
-If there is a conflict during a merge, 3 files with the same basename are created. The filename of the common base is appended with .BASE, the filename of the file containing your changes is appended .THIS and the filename with the changes from the other tree is appended .OTHER.
-Using a program such as kdiff3, you can now comfortably merge them into one file. To commit you have to rename it to the original basename and delete the other two files. As long as there exist files with .BASE, .THIS or .OTHER the commit command will complain.
+If there is a conflict during a merge, 3 files with the same basename are
+created. The filename of the common base is appended with .BASE, the
+filename of the file containing your changes is appended .THIS and the
+filename with the changes from the other tree is appended .OTHER.
+Using a program such as kdiff3, you can now comfortably merge them into
+one file. To commit you have to rename it to the original basename and
+delete the other two files. As long as there exist files with .BASE, .THIS
+or .OTHER the commit command will complain.
 
 [**TODO**: explain conflict markers within files]
 
 
 Publishing your branch
 ======================
-You don't need a special server to publish a bzr branch, just a normal web server.  Just mirror the files to your server, including the .bzr directory.  One can push a branch (or the changes for a branch) by one of the following three methods:
-
- * Rsync:   rsync -avrz LOCALBRANCH  servername.com/this/directory/here
- * (Post 0.6): bzr push sftp://servername.com/this/directory/here (sftp://servername.com/this/directory/ must already exist)
- * The push plugin that comes with BzrTools
+
+You don't need a special server to publish a bzr branch, just a normal web
+server.  Just mirror the files to your server, including the .bzr
+directory.  One can push a branch (or the changes for a branch) by one of
+the following three methods:
+
+* Rsync:   rsync -avrz LOCALBRANCH  servername.com/this/directory/here
+
+  (or any other tool for publishing a directory to a web site.)
+
+* bzr push sftp://servername.com/this/directory/here 
+
+  (The directory that must already exist)
+
+* The push plugin that comes with BzrTools
 

